package com.ibm.ccl.ua.generators.sitexml;

/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.eclipse.ant.core.AntRunner;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.osgi.framework.Bundle;
/**
 * Servlet to handle live help action requests
 */
public class PluginReader extends HttpServlet {
	private static final long serialVersionUID = 1L;
	private static final String BUILD_FILE = "buildfeature.xml";
	public final String JAR_EXTENSION = ".jar";

	/**
	 */
	public void init() throws ServletException {
		
	}

	/**
	 * Called by the server (via the <code>service</code> method) to allow a
	 * servlet to handle a GET request.
	 */
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		

	    resp.setContentType("application/java-archive"); 
	    
		//Get the Plug-in Path
		String pluginPath = req.getPathInfo();
		
		//Trim first slash to get feature name
		String pluginRequested=pluginPath.substring(1);
		
		boolean jarredPluginFound=true;
		
		// Get path to plug-in
		String workspaceLocation=getWorkspaceLocation();
		String eclipseLocation = getEclipseLocation(workspaceLocation);
		String pluginDirPath = eclipseLocation + "eclipse" + File.separator + "plugins";
		String filepath = eclipseLocation + "eclipse" + File.separator + "plugins" + File.separator + pluginRequested;
		
		if (!new File(filepath).exists()){
			jarredPluginFound=false;
			
			AntUtil.setFeatureName(pluginRequested.substring(0, pluginRequested.indexOf(JAR_EXTENSION)));
			// Source and Destination of the plug-ins is the same
			AntUtil.setFeatureSource(pluginDirPath);
			AntUtil.setFeatureDestination(pluginDirPath);
		
			try {
				/*
				 * Temporarily copy the buildfeature.xml file to the workspace
				 * and pass its location on disk to the
				 * setBuildFileLocation(String) method of the AntRunner class.
				 */
				String BuildFilePath = workspaceLocation + File.separator
						+ BUILD_FILE;
				copyPluginResource(Activator.getDefault().PLUGIN_ID,
						BUILD_FILE, new File(BuildFilePath));
	
				AntRunner runner = new AntRunner();
				// Set the build file location to point to our build file
				runner.setBuildFileLocation(BuildFilePath);
				// Execute the build to jar the feature
				runner.run();
			} catch (CoreException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			
		}
       
        File file = new File(filepath);
        resp.setContentLength((int)file.length());       
        
		// Open the file and output streams
        FileInputStream in = new FileInputStream(file);
		OutputStream out = resp.getOutputStream();

    	// Copy the contents of the file to the output stream
    	byte[] buf = new byte[1024];
    	int count = 0;
    	while ((count = in.read(buf)) >= 0) {
    		out.write(buf, 0, count);
    	}
    	in.close();
    	out.close();
	
		//Delete the jar file from the IC - if created.
		if (!jarredPluginFound){
			boolean result = new File(pluginDirPath + File.separator + pluginRequested).delete();
			if(result)
				System.out.println("The plug-in " + pluginRequested + " has been removed from the IC");
		}
				
	}
	/**
	 * 
	 * Called by the server (via the <code>service</code> method) to allow a
	 * servlet to handle a POST request.
	 * 
	 * Handle the search requests,
	 *  
	 */
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		doGet(req, resp);
	}
	
	
	public String getWorkspaceLocation()
	{
		IWorkspace userWorkspace = org.eclipse.core.resources.ResourcesPlugin.getWorkspace();
		String workspaceLocation=userWorkspace.getRoot().getLocation().toOSString();
		
		return workspaceLocation;
	}
	
	public String getEclipseLocation(String workspaceLoc)
	{
		String eclipseLoc;
		
		int eclipseIndex=workspaceLoc.lastIndexOf("eclipse");
		
		eclipseLoc=workspaceLoc.substring(0, eclipseIndex);
		
		return eclipseLoc;
	}
	/**
	 * Copies a particular resource from a Bundle into the selected destination
	 * file
	 * 
	 * @param pluginID
	 *            Bundle ID where the resource is located
	 * @param resource
	 *            Name of the resource to copy
	 * @param dst
	 *            Destination file to which to copy the resource
	 * @return True if the Bundle is found, false otherwise
	 * @throws IOException
	 */
	
	public boolean copyPluginResource(String pluginID, String resource, File dst)
			throws IOException {

		InputStream in = null;

		try {
			Bundle bundle = Platform.getBundle(pluginID);

			if (bundle == null)
				return false;

			Path path = new Path(resource);
			URL fileURL = FileLocator.find(bundle, path, null);
			in = fileURL.openStream();

		} catch (IOException e) {
			e.printStackTrace();
		}
		OutputStream out = new FileOutputStream(dst);

		// Transfer bytes from in to out
		byte[] buf = new byte[2156];
		int len;
		while ((len = in.read(buf)) > 0) {
			out.write(buf, 0, len);
		}
		in.close();
		out.close();
		return true;
	}
}

